class DropdownMenu extends HTMLElement {
  constructor() {
    super();
  }
  get open() {
    const ul = this.shadowRoot.querySelector('ul');
    return ul.hasAttribute('open');
  }
  set open(val) {
    const ul = this.shadowRoot.querySelector('ul');

    if (val) {
      ul.setAttribute('open', '');
    } else {
      ul.removeAttribute('open');
    }
  }

  _toggleDrawer() {
    if (this.open) {
      this.open = false;
    } else {
      this.open = true;
    }
  }

  _closeDrawer() {
    this.open = false;
  }

  _chooseOption(val) {
    const span = this.shadowRoot.querySelector('span');
    span.textContent = val;
  }

  connectedCallback() {
    let placeholder = this.getAttribute('placeholder') || 'Select an option';
    let template = document.createElement('template');
    template.innerHTML = `
      <style>
        .c-dropdown {
          position: relative;
          height: 35px;
          background: #FFF;
          border: 1px solid #BBB;
          border-radius: 3px;
          cursor: pointer;
        }
        span {
          display: block;
          margin: 0 2em;
          line-height: 35px;
          height: 35px;
          overflow: hidden;
        }
        ul {
          z-index: 1000;
          list-style: none;
          background: #FFF;
          border: 1px solid rgba(0,0,0,0.15);
          border-radius: 3px;
          -webkit-box-shadow: 0 6px 12px rgba(0,0,0,0.175);
          -moz-box-shadow: 0 6px 12px rgba(0,0,0,0.175);
          box-shadow: 0 6px 12px rgba(0,0,0,0.175);
          -webkit-background-clip: padding-box;
          -moz-background-clip: padding-box;
          background-clip: padding-box;
          max-height: 270px;
          overflow: scroll;
          margin: 0;
          padding: 0;
          display: none;
        }
        ::slotted(li) {
          padding: .5em 2em;
          text-align: left;
        }
        ::slotted(li:hover) {
          background: #eee;
          cursor: pointer;
        }
        ul[open] {
          display: block;
        }
      </style>
      <div class="c-dropdown">
        <span>${placeholder}</span>
        <i class="caret caret--blue"></i>
        <ul class="c-dropdown-menu">
          <slot><li></li></slot>
        </ul>
      </div>`;
    let shadowRoot = this.attachShadow({mode: 'open'});
    shadowRoot.appendChild(template.content.cloneNode(true));
    document.addEventListener('click', (e) => {
      if (e.target.tagName === 'LI' && this.contains(e.target)) {
        this._chooseOption(e.target.textContent);
        this._toggleDrawer();
      } else if (this.contains(e.target)) {
        this._toggleDrawer();
      } else {
        this._closeDrawer();
      }
    });
    this._initDropDown();
  }
  _initDropDown() {
    this.addEventListener('click', (e) => {
      if (e.target.tagName === 'LI') {
        let event = new CustomEvent('initDropdown', {
          detail: {
            selectedValue: e.target.textContent,
            selectedNode: e.target
          }
        });
        this.dispatchEvent(event);
      }
    });
  }
}
customElements.define('dropdown-menu', DropdownMenu);
